/* XPiezo - a tool to configure the X3D-Piezo Gyro
 * Copyright (C) 2006 Simon Budig <simon@budig.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include "xpiezo.h"

#include "xpiezo-icons.h"

#define _(arg) arg
#define N_(arg) arg

static const gchar xpiezo_ui[] =
   "<ui>"
   "  <menubar>"
   "    <menu name=\"FileMenu\" action=\"xpiezo-file-menu\">"
   "      <menuitem action=\"xpiezo-config-open\" />"
   "      <menuitem action=\"xpiezo-config-save\" />"
   "      <menuitem action=\"xpiezo-config-defaults\" />"
   "      <separator/>"
   "      <menuitem action=\"xpiezo-load-from-x3d\" />"
   "      <menuitem action=\"xpiezo-save-to-x3d\" />"
   "      <separator/>"
   "      <menuitem action=\"xpiezo-quit\" />"
   "    </menu>"
   "    <menu name=\"PresetMenu\" action=\"xpiezo-preset-menu\">"
   "      <separator/>"
   "      <menuitem action=\"xpiezo-preset-1\" />"
   "      <menuitem action=\"xpiezo-preset-2\" />"
   "      <menuitem action=\"xpiezo-preset-3\" />"
   "      <menuitem action=\"xpiezo-preset-4\" />"
   "      <separator/>"
   "      <menuitem action=\"xpiezo-preset-import\" />"
   "      <menuitem action=\"xpiezo-preset-export\" />"
   "      <separator/>"
   "      <menuitem action=\"xpiezo-preset-reorder\" />"
   "    </menu>"
   "  </menubar>"
   "  <!-- <toolbar action=\"xpiezo-toolbar\">"
   "    <toolitem action=\"xpiezo-load-from-x3d\"/>"
   "    <toolitem action=\"xpiezo-save-to-x3d\"/>"
   "    <separator/>"
   "    <toolitem action=\"xpiezo-quit\"/>"
   "  </toolbar> -->"
   "</ui>"
;


static void
xpiezo_orientation_callback (GtkRadioButton *radio,
                             gpointer        user_data)
{
  XPiezoContext *context = (XPiezoContext *) user_data;
  gint dir;

  if (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (radio)))
    return;

  dir = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (radio), "direction"));

  gtk_image_set_from_pixbuf (GTK_IMAGE (context->ufo_image),
                             context->ufo_pixbufs[dir]);

  gtk_adjustment_set_value (param_defs[XPIEZO_ORIENTATION].adj, dir);
}

static void
xpiezo_orientation_changed (GtkAdjustment *adj,
                            XPiezoContext *context)
{
  XPiezoPreset *preset;
  GtkWidget *ori_button;
  gdouble val;

  preset = &context->presets[context->cur_preset];
  val = gtk_adjustment_get_value (adj);

  preset->orientation = (gint) (val + 0.5);
  ori_button = context->ori_buttons[preset->orientation];
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (ori_button), TRUE);
}

static void
xpiezo_fast_yaw_callback (GtkToggleButton *toggle,
                          gpointer         user_data)
{
  gboolean fy ;

  fy = gtk_toggle_button_get_active (toggle);

  gtk_adjustment_set_value (param_defs[XPIEZO_YAW_FAST_YAW].adj, fy ? 1.0 : 0.0);
}

static void
xpiezo_fast_yaw_changed (GtkAdjustment *adj,
                         XPiezoContext *context)
{
  XPiezoPreset *preset;
  GtkWidget *fast_yaw_toggle;
  gdouble val;

  preset = &context->presets[context->cur_preset];
  val = gtk_adjustment_get_value (adj);

  preset->fast_yaw = val > 0.5 ? TRUE : FALSE;
  fast_yaw_toggle = context->fast_yaw_toggle;
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (fast_yaw_toggle),
                                preset->fast_yaw);
}

static void
xpiezo_complex_value_changed (GtkAdjustment *adj,
                              XPiezoContext *context)
{
  XPiezoPreset *preset;
  gdouble val;
  gint s_offset;

  if (context->in_update)
    return;

  s_offset = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (adj), "s_offset"));
  preset = &context->presets[context->cur_preset];
  val = gtk_adjustment_get_value (adj);

  preset->easyparams = FALSE;
  G_STRUCT_MEMBER (gint, preset, s_offset) = (gint) (val + 0.5);
}

static void
xpiezo_easy_value_changed (GtkAdjustment *adj,
                           XPiezoContext *context)
{
  static GtkWidget *dialog = NULL;
  XPiezoPreset *preset;
  gint val, response;
  gint s_offset;

  if (context->in_update)
    return;

  s_offset = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (adj), "s_offset"));
  preset = &context->presets[context->cur_preset];
  val = (gint) (gtk_adjustment_get_value (adj) + 0.5);

  if (G_STRUCT_MEMBER (gint, preset, s_offset) != val &&
      preset->easyparams == FALSE)
    {
      if (!dialog)
        {
          dialog = gtk_message_dialog_new_with_markup
                             (GTK_WINDOW (context->main_window),
                              GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
                              GTK_MESSAGE_QUESTION,
                              GTK_BUTTONS_YES_NO,
                              "<b>Switch to easy parameters?</b>\n"
                              "\n"
                              "These parameters will affect the parameters "
                              "in the <i>expert</i> section of the preset. "
                              "Do you want to switch the preset to the "
                              "simplified mode?");
        }

      response = gtk_dialog_run (GTK_DIALOG (dialog));
      switch (response)
        {
          case GTK_RESPONSE_YES:
            preset->easyparams = TRUE;
            G_STRUCT_MEMBER (gint, preset, s_offset) = (gint) (val + 0.5);
            break;
          default:
            gtk_adjustment_set_value (adj,
                                      G_STRUCT_MEMBER (gint, preset, s_offset));

        }
      gtk_widget_hide (dialog);
    }
  else
    {
      G_STRUCT_MEMBER (gint, preset, s_offset) = (gint) (val + 0.5);
    }

  xpiezo_preset_update_from_easy (context);
}

static void
xpiezo_combo_callback (GtkComboBox   *box,
                       XPiezoContext *context)
{
  gint i;

  i = gtk_combo_box_get_active (box);

  if (context->cur_preset != i)
    gtk_action_activate (context->preset_actions[i]);
}

static void
xpiezo_create_statusbar (XPiezoContext *context,
                         GtkWidget     *parent)
{
  GtkWidget *image, *frame, *statusbar;

  statusbar = g_object_new (GTK_TYPE_STATUSBAR,
                            "homogeneous", FALSE,
                            "visible", TRUE,
                            NULL);
  gtk_box_pack_end (GTK_BOX (parent), statusbar, FALSE, FALSE, 0);

  frame = g_object_new (GTK_TYPE_FRAME,
                        "shadow", GTK_SHADOW_IN,
                        "visible", TRUE,
                        NULL);
  image = g_object_new (GTK_TYPE_IMAGE,
                        "stock", "gtk-dialog-warning",
                        "xpad", 1,
                        "ypad", 1,
                        "parent", frame,
                        "visible", TRUE,
                        NULL);
  gtk_box_pack_start (GTK_BOX (statusbar), frame, FALSE, FALSE, 0);
  gtk_box_reorder_child (GTK_BOX (statusbar), frame, 0);

  frame = g_object_new (GTK_TYPE_FRAME,
                        "shadow", GTK_SHADOW_IN,
                        "visible", TRUE,
                        NULL);
  image = g_object_new (GTK_TYPE_IMAGE,
                        "stock", "gtk-connect",
                        "xpad", 1,
                        "ypad", 1,
                        "parent", frame,
                        "visible", TRUE,
                        NULL);
  gtk_box_pack_start (GTK_BOX (statusbar), frame, FALSE, FALSE, 0);
  gtk_box_reorder_child (GTK_BOX (statusbar), frame, 0);
}

static void
xpiezo_create_orientation (XPiezoContext *context,
                           GtkWidget     *parent)
{
  GtkWidget *label, *frame, *alignment, *table, *radio, *image;
  GtkObject *adj;
  GdkPixbuf *ufo0, *ufo45, *arrow, *tmp;
  GSList *rgroup;
  gint rcoords[] = { 2,0, 1,1, 0,2, 1,3, 2,4, 3,3, 4,2, 3,1 };
  gint i;

  g_return_if_fail (context->ufo_pixbufs == NULL);

  context->ufo_pixbufs = g_new (GdkPixbuf *, 8);
  ufo0 =  gdk_pixbuf_new_from_inline (-1, ufo0_inline, FALSE, NULL);
  ufo45 = gdk_pixbuf_new_from_inline (-1, ufo45_inline, FALSE, NULL);
  arrow = gdk_pixbuf_new_from_inline (-1, arrow_inline, FALSE, NULL);

  for (i = 0; i < 8; i++)
    {
      tmp = gdk_pixbuf_rotate_simple ((i % 2) == 0 ? ufo0 : ufo45,
                                      (360 + (i / 2) * 90) % 360);
      gdk_pixbuf_composite (arrow, tmp, 0, 0, 80, 80,
                            (i > 4) ? -24 : 24, 0,
                            1.0, 1.0, GDK_INTERP_NEAREST, 255);
      context->ufo_pixbufs[i] = tmp;
    }

  g_object_unref (ufo0);
  g_object_unref (ufo45);
  g_object_unref (arrow);

  label = g_object_new (GTK_TYPE_LABEL,
                        "label", param_defs[XPIEZO_ORIENTATION].label,
                        "use-markup", TRUE,
                        "visible", TRUE,
                        NULL);


  frame = g_object_new (GTK_TYPE_FRAME,
                        "label-widget", label,
                        "shadow-type", GTK_SHADOW_NONE,
                        "visible", TRUE,
                        NULL);
  gtk_box_pack_start (GTK_BOX (parent), frame, FALSE, FALSE, 5);

  alignment = g_object_new (GTK_TYPE_ALIGNMENT,
                            "left-padding", 12,
                            "parent", frame,
                            "visible", TRUE,
                            NULL);

  table = g_object_new (GTK_TYPE_TABLE,
                        "n-rows", 5,
                        "n-columns", 5,
                        "parent", alignment,
                        "visible", TRUE,
                        NULL);

  context->ori_buttons = g_new (GtkWidget *, 8);

  rgroup = NULL;

  for (i = 0; i < 8; i++)
    {
      radio = gtk_radio_button_new (rgroup);
      context->ori_buttons[i] = radio;
      gtk_widget_show (radio);

      g_object_set_data (G_OBJECT (radio), "direction", GINT_TO_POINTER (i));
      g_signal_connect (radio, "toggled",
                        G_CALLBACK (xpiezo_orientation_callback),
                        context);

      gtk_table_attach (GTK_TABLE (table), radio,
                        rcoords [i*2], rcoords[i*2] + 1,
                        rcoords [i*2+1], rcoords[i*2+1] + 1,
                        0, 0, 0, 0);
      rgroup = gtk_radio_button_get_group (GTK_RADIO_BUTTON (radio));
    }

  adj = gtk_adjustment_new (0, 0, 7, 1, 1, 0);
  g_object_set_data (G_OBJECT (adj), "s_offset",
                     GINT_TO_POINTER (param_defs[XPIEZO_ORIENTATION].s_offset));
  g_signal_connect (adj, "value_changed",
                    G_CALLBACK (xpiezo_orientation_changed), context);
  param_defs[XPIEZO_ORIENTATION].adj = GTK_ADJUSTMENT (adj);

  image = g_object_new (GTK_TYPE_IMAGE,
                        "pixbuf", context->ufo_pixbufs[0],
                        "visible", TRUE,
                        "height-request", 80,
                        "width-request", 80,
                        NULL);

  context->ufo_image = image;

  gtk_table_attach (GTK_TABLE (table), image, 2, 3, 2, 3, 0, 0, 0, 0);
}

static void
xpiezo_create_easy_params (XPiezoContext *context,
                           GtkWidget     *parent)
{
  GtkWidget *table, *hbox, *label, *scale, *button;
  GtkObject *adj;

  table = g_object_new (GTK_TYPE_TABLE,
                        "n-rows", 5,
                        "n-columns", 4,
                        "visible", TRUE,
                        NULL);

  gtk_box_pack_start (GTK_BOX (parent), table, FALSE, FALSE, 5);

  hbox = g_object_new (GTK_TYPE_HBOX,
                       "spacing", 12,
                       "homogeneous", TRUE,
                       "visible", TRUE,
                       NULL);

  label = g_object_new (GTK_TYPE_LABEL,
                        "label", _("<i><small>lightweight</small></i>"),
                        "use-markup", TRUE,
                        "xalign", 0.0,
                        "justify", GTK_JUSTIFY_LEFT,
                        "parent", hbox,
                        "visible", TRUE,
                        NULL);
  label = g_object_new (GTK_TYPE_LABEL,
                        "label", _("<i><small>medium</small></i>"),
                        "use-markup", TRUE,
                        "xalign", 0.5,
                        "justify", GTK_JUSTIFY_CENTER,
                        "parent", hbox,
                        "visible", TRUE,
                        NULL);
  label = g_object_new (GTK_TYPE_LABEL,
                        "label", _("<i><small>heavy</small></i>"),
                        "use-markup", TRUE,
                        "xalign", 1.0,
                        "justify", GTK_JUSTIFY_RIGHT,
                        "parent", hbox,
                        "visible", TRUE,
                        NULL);
  gtk_table_attach (GTK_TABLE (table), hbox, 1, 4, 0, 1,
                    GTK_FILL, 0, 0, 0);

  label = g_object_new (GTK_TYPE_LABEL,
                        "label", param_defs[XPIEZO_EASY_WEIGHT].label,
                        "xalign", 0.0,
                        "visible", TRUE,
                        NULL);
  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2,
                    GTK_FILL, 0, 0, 0);

  adj = gtk_adjustment_new (0, 0, 7, 1, 1, 0);
  g_object_set_data (G_OBJECT (adj), "s_offset",
                     GINT_TO_POINTER (param_defs[XPIEZO_EASY_WEIGHT].s_offset));
  g_signal_connect (adj, "value_changed",
                    G_CALLBACK (xpiezo_easy_value_changed), context);
  param_defs[XPIEZO_EASY_WEIGHT].adj = GTK_ADJUSTMENT (adj);

  scale = g_object_new (GTK_TYPE_HSCALE,
                        "update-policy", GTK_UPDATE_DISCONTINUOUS,
                        "draw-value", FALSE,
                        "adjustment", adj,
                        "visible", TRUE,
                        NULL);
  GTK_RANGE (scale)->round_digits = 0;

  gtk_table_attach (GTK_TABLE (table), scale, 1, 4, 1, 2,
                    GTK_EXPAND | GTK_FILL, 0, 0, 0);

  label = g_object_new (GTK_TYPE_LABEL,
                        "label", param_defs[XPIEZO_EASY_AGILITY].label,
                        "xalign", 0.0,
                        "visible", TRUE,
                        NULL);
  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 2, 3,
                    GTK_FILL, 0, 0, 0);

  adj = gtk_adjustment_new (0, 0, 7, 1, 1, 0);
  g_object_set_data (G_OBJECT (adj), "s_offset",
                     GINT_TO_POINTER (param_defs[XPIEZO_EASY_AGILITY].s_offset));
  g_signal_connect (adj, "value_changed",
                    G_CALLBACK (xpiezo_easy_value_changed), context);
  param_defs[XPIEZO_EASY_AGILITY].adj = GTK_ADJUSTMENT (adj);

  scale = g_object_new (GTK_TYPE_HSCALE,
                        "update-policy", GTK_UPDATE_DISCONTINUOUS,
                        "draw-value", FALSE,
                        "adjustment", adj,
                        "visible", TRUE,
                        NULL);
  gtk_table_attach (GTK_TABLE (table), scale, 1, 4, 2, 3,
                    GTK_EXPAND | GTK_FILL, 0, 0, 0);

  GTK_RANGE (scale)->round_digits = 0;

  hbox = g_object_new (GTK_TYPE_HBOX,
                       "spacing", 12,
                       "homogeneous", TRUE,
                       "visible", TRUE,
                       NULL);

  label = g_object_new (GTK_TYPE_LABEL,
                        "label", _("<i><small>lazy, slow,\nindoor</small></i>"),
                        "use-markup", TRUE,
                        "xalign", 0.0,
                        "justify", GTK_JUSTIFY_LEFT,
                        "parent", hbox,
                        "visible", TRUE,
                        NULL);
  label = g_object_new (GTK_TYPE_LABEL,
                        "label", _("<i><small>agile, vivid,\noutdoor</small></i>"),
                        "use-markup", TRUE,
                        "xalign", 1.0,
                        "justify", GTK_JUSTIFY_RIGHT,
                        "parent", hbox,
                        "visible", TRUE,
                        NULL);
  gtk_table_attach (GTK_TABLE (table), hbox, 1, 4, 3, 4,
                    GTK_EXPAND | GTK_FILL, 0, 0, 0);

  label = g_object_new (GTK_TYPE_LABEL,
                        "label", param_defs[XPIEZO_YAW_FAST_YAW].label,
                        "xalign", 0.0,
                        "visible", TRUE,
                        NULL);
  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 4, 5,
                    GTK_FILL, 0, 0, 0);

  adj = gtk_adjustment_new (0, 0, 1, 1, 1, 0);
  g_object_set_data (G_OBJECT (adj), "s_offset",
                     GINT_TO_POINTER (param_defs[XPIEZO_YAW_FAST_YAW].s_offset));
  g_signal_connect (adj, "value_changed",
                    G_CALLBACK (xpiezo_fast_yaw_changed), context);
  param_defs[XPIEZO_YAW_FAST_YAW].adj = GTK_ADJUSTMENT (adj);

  button = g_object_new (GTK_TYPE_CHECK_BUTTON,
                         "active", FALSE,
                         "visible", TRUE,
                         NULL);

  g_signal_connect (button, "toggled",
                    G_CALLBACK (xpiezo_fast_yaw_callback), context);

  context->fast_yaw_toggle = button;

  gtk_table_attach (GTK_TABLE (table), button, 1, 5, 4, 5,
                    GTK_FILL, 0, 0, 0);

}

static void
xpiezo_create_complex_params (XPiezoContext *context,
                              GtkWidget     *parent)
{
  GtkWidget *expander, *table, *label, *alignment, *spinbutton;
  GtkObject *adj;
  gint i, row, col;

  expander = g_object_new (GTK_TYPE_EXPANDER,
                           "label", "<b>Expert parameters</b>",
                           "use-markup", TRUE,
                           "expanded", FALSE,
                           "visible", TRUE,
                           NULL);
  gtk_box_pack_start (GTK_BOX (parent), expander, FALSE, FALSE, 5);

  alignment = g_object_new (GTK_TYPE_ALIGNMENT,
                            "left-padding", 12,
                            "parent", expander,
                            "visible", TRUE,
                            NULL);

  table = g_object_new (GTK_TYPE_TABLE,
                        "n-rows", 8,
                        "n-columns", 4,
                        "parent", alignment,
                        "visible", TRUE,
                        NULL);

  label = g_object_new (GTK_TYPE_LABEL,
                        "label", "Nick/Roll",
                        "visible", TRUE,
                        "xalign", 0.0,
                        NULL);
  gtk_table_attach (GTK_TABLE (table), label, 0, 2, 0, 1,
                    GTK_FILL, 0, 0, 0);
  label = g_object_new (GTK_TYPE_LABEL,
                        "label", "Yaw",
                        "visible", TRUE,
                        "xalign", 0.0,
                        NULL);
  gtk_table_attach (GTK_TABLE (table), label, 2, 4, 0, 1,
                    GTK_FILL, 0, 0, 0);

  col = 0;
  row = 1;

  for (i = XPIEZO_NR_KP; i < XPIEZO_PARAM_LAST; i++)
    {
      if (i == XPIEZO_YAW_FAST_YAW)
        {
          i++;
        }

      if (i == XPIEZO_YAW_KP)
        {
          col += 2;
          row = 1;
        }

      label = g_object_new (GTK_TYPE_LABEL,
                            "label", param_defs[i].label,
                            "use-markup", TRUE,
                            "xpad", 12,
                            "visible", TRUE,
                            "xalign", 0.0,
                            NULL);
      gtk_table_attach (GTK_TABLE (table), label,
                        col, col + 1, row, row+1,
                        GTK_FILL, 0, 0, 0);

      adj = gtk_adjustment_new (param_defs[i].factory[0],
                                param_defs[i].min,
                                param_defs[i].max,
                                1, 1, 0);
      g_object_set_data (G_OBJECT (adj), "s_offset",
                         GINT_TO_POINTER (param_defs[i].s_offset));
      g_signal_connect (adj, "value_changed",
                        G_CALLBACK (xpiezo_complex_value_changed), context);
      param_defs[i].adj = GTK_ADJUSTMENT (adj);

      spinbutton = g_object_new (GTK_TYPE_SPIN_BUTTON,
                                 "adjustment", adj,
                                 "digits", 0,
                                 "visible", TRUE,
                                 NULL);
      gtk_table_attach (GTK_TABLE (table), spinbutton,
                        col + 1, col + 2, row, row + 1,
                        GTK_FILL | GTK_EXPAND, 0, 0, 0);

      if (i == XPIEZO_TEMP_COMP)
        break;

      row += 1;

      if (i == XPIEZO_YAW_FAST_MARGIN)
        {
          row += 1;
          i = XPIEZO_TEMP_COMP - 1;
        }

    }
}

void
xpiezo_refresh_gui (XPiezoContext *context)
{
  XPiezoPreset preset;
  gint i, val;

  preset = context->presets[context->cur_preset];

  context->in_update = TRUE;

  for (i = XPIEZO_EASY_WEIGHT; i < XPIEZO_PARAM_LAST; i++)
    {
      if (param_defs[i].adj)
        {
          val = G_STRUCT_MEMBER (gint, &preset, param_defs[i].s_offset);
          gtk_adjustment_set_value (param_defs[i].adj, val);
        }
    }

  context->in_update = FALSE;
}

GtkWidget *
xpiezo_create_window (XPiezoContext *context)
{
  GtkUIManager   *ui_manager;
  GtkActionGroup *action_group;
  GtkAccelGroup  *accel_group;
  GtkWidget      *window, *main_vbox, *vbox,
                 *ui_widget, *label, *alignment, *frame;
  GtkAction      *act;
  GSList         *list;
  GError *error = NULL;
  gint i;
  gchar *action_label;

  window = g_object_new (GTK_TYPE_WINDOW,
                         "type", GTK_WINDOW_TOPLEVEL,
                         "title", "xpiezo",
                         "default-height", 100,
                         "default-width", 100,
                         NULL);
  g_signal_connect (window, "delete_event", gtk_main_quit, NULL);

  main_vbox = g_object_new (GTK_TYPE_VBOX,
                            "parent", window,
                            "spacing", 0,
                            "visible", TRUE,
                            NULL);

  action_group = xpiezo_get_actions (context);

  ui_manager = gtk_ui_manager_new ();
  gtk_ui_manager_insert_action_group (ui_manager, action_group, 0);
  gtk_ui_manager_add_ui_from_string (ui_manager, xpiezo_ui, -1, &error);

  if (error)
    g_printerr ("UI-Manager: %s\n", error->message);

  g_clear_error (&error);

  ui_widget = gtk_ui_manager_get_widget (ui_manager, "ui/menubar");
  gtk_box_pack_start (GTK_BOX (main_vbox), ui_widget, FALSE, FALSE, 0);

  ui_widget = gtk_ui_manager_get_widget (ui_manager, "ui/xpiezo-toolbar");
  if (ui_widget)
    {
      gtk_toolbar_set_style (GTK_TOOLBAR (ui_widget), GTK_TOOLBAR_ICONS);
      gtk_box_pack_start (GTK_BOX (main_vbox), ui_widget, FALSE, FALSE, 0);
    }
  accel_group = gtk_ui_manager_get_accel_group (ui_manager);
  gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);

  ui_widget = gtk_combo_box_new_text ();
  act = gtk_ui_manager_get_action (ui_manager,
                                   "ui/menubar/PresetMenu/xpiezo-preset-1");
  list = gtk_radio_action_get_group (GTK_RADIO_ACTION (act));

  for (; list; list = g_slist_next (list))
    {
      g_object_get (list->data, "value", &i, NULL);
      context->preset_actions[i] = GTK_ACTION (list->data);;
    }

  for (i = 0; i < 4; i++)
    {
      g_object_get (context->preset_actions[i], "label", &action_label, NULL);
      gtk_combo_box_append_text (GTK_COMBO_BOX (ui_widget), action_label);
    }

  g_signal_connect (ui_widget, "changed",
                    G_CALLBACK (xpiezo_combo_callback),
                    context);

  gtk_combo_box_set_active (GTK_COMBO_BOX (ui_widget), 0);

  gtk_widget_show (ui_widget);
  gtk_box_pack_start (GTK_BOX (main_vbox), ui_widget, FALSE, FALSE, 5);

  context->preset_button = ui_widget;

  xpiezo_create_orientation (context, main_vbox);

  /* Parameterarea: Easy parameters */

  label = g_object_new (GTK_TYPE_LABEL,
                        "label", _("<b>Parameters</b>"),
                        "use-markup", TRUE,
                        "visible", TRUE,
                        NULL);

  frame = g_object_new (GTK_TYPE_FRAME,
                        "label-widget", label,
                        "shadow-type", GTK_SHADOW_NONE,
                        "visible", TRUE,
                        NULL);
  gtk_box_pack_start (GTK_BOX (main_vbox), frame, FALSE, FALSE, 5);

  alignment = g_object_new (GTK_TYPE_ALIGNMENT,
                            "left-padding", 12,
                            "parent", frame,
                            "visible", TRUE,
                            NULL);

  vbox = g_object_new (GTK_TYPE_VBOX,
                       "parent", alignment,
                       "spacing", 10,
                       "visible", TRUE,
                       NULL);

  xpiezo_create_easy_params (context, vbox);

  xpiezo_create_complex_params (context, main_vbox);

  xpiezo_create_statusbar (context, main_vbox);

  return window;
}

