/* XPiezo - a tool to configure the X3D-Piezo Gyro
 * Copyright (C) 2006 Simon Budig <simon@budig.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <gtk/gtk.h>
#include "xpiezo.h"

/* definitions for the parameters of a preset */
ParamPreset param_defs[] = {
     /*  enum                     label                   default 1,    2,       3,       4    min    max         struct_offset                          adjustment  */
    { XPIEZO_EASYPARAMS,        "Easyparams:",          {       1,       1,       1,       0, }, 0,       1, G_STRUCT_OFFSET (XPiezoPreset, easyparams),        NULL },
    { XPIEZO_EASY_WEIGHT,       "Weight:",              {       5,       2,       2,       0, }, 0,       7, G_STRUCT_OFFSET (XPiezoPreset, weight),            NULL },
    { XPIEZO_EASY_AGILITY,      "Agility:",             {       0,       2,       4,       7, }, 0,       7, G_STRUCT_OFFSET (XPiezoPreset, agility),           NULL },

    { XPIEZO_ORIENTATION,       "<b>Orientation</b>",   {       0,       0,       0,       0, }, 0,       7, G_STRUCT_OFFSET (XPiezoPreset, orientation),       NULL },
    { XPIEZO_TEMP_COMP,         "Temp. comp.",          {      50,      50,      50,      50  }, 0,    1000, G_STRUCT_OFFSET (XPiezoPreset, temp_comp),         NULL },

    { XPIEZO_NR_KP,             "K<sub>P</sub>:",       {     220,     178,     178,     180  }, 0,   10000, G_STRUCT_OFFSET (XPiezoPreset, nr_kp),             NULL },
    { XPIEZO_NR_KD,             "K<sub>D</sub>:",       {     470,     428,     428,     470  }, 0,   10000, G_STRUCT_OFFSET (XPiezoPreset, nr_kd),             NULL },
    { XPIEZO_NR_ANGLE_MAX,      "angle<sub>max</sub>:", { 1000000, 2000000, 2000000, 2000000  }, 0, 5000000, G_STRUCT_OFFSET (XPiezoPreset, angle_max),         NULL },
    { XPIEZO_NR_K_STICK,        "K_stick:",             {       8,      12,      16,      20  }, 0,     100, G_STRUCT_OFFSET (XPiezoPreset, nr_k_stick),        NULL },
    { XPIEZO_NR_K_STICK_DIRECT, "K_stick_direct:",      {      10,      11,      12,      14  }, 0,    1000, G_STRUCT_OFFSET (XPiezoPreset, nr_k_stick_direct), NULL },
    { XPIEZO_NR_K_EXPO,         "K_expo:",              {      60,      86,     112,     150  }, 0,    1000, G_STRUCT_OFFSET (XPiezoPreset, nr_k_expo),         NULL },
    { XPIEZO_NR_K_EXPO_EDGE,    "K_expo_edge:",         {    1000,    5000,    9000,   12000  }, 0,   10000, G_STRUCT_OFFSET (XPiezoPreset, nr_k_expo_edge),    NULL },

    { XPIEZO_YAW_KP,            "K<sub>P</sub>:",       {     410,     392,     392,     400  }, 0,   10000, G_STRUCT_OFFSET (XPiezoPreset, yaw_kp),            NULL },
    { XPIEZO_YAW_KD,            "K<sub>D</sub>:",       {     535,     514,     514,     540  }, 0,   10000, G_STRUCT_OFFSET (XPiezoPreset, yaw_kd),            NULL },
    { XPIEZO_YAW_K_STICK,       "K_stick:",             {      20,      24,      28,      40  }, 0,     100, G_STRUCT_OFFSET (XPiezoPreset, yaw_k_stick),       NULL },
    { XPIEZO_YAW_FAST_YAW,      "Fast yaw:",            {       0,       1,       1,       1  }, 0,       1, G_STRUCT_OFFSET (XPiezoPreset, fast_yaw),          NULL },
    { XPIEZO_YAW_K_FAST,        "K_fast_yaw:",          {      60,      82,     104,     110  }, 0,   10000, G_STRUCT_OFFSET (XPiezoPreset, k_fast_yaw),        NULL },
    { XPIEZO_YAW_FAST_MARGIN,   "fast_yaw_margin:",     {      15,      15,      15,      10  }, 0,    1000, G_STRUCT_OFFSET (XPiezoPreset, fast_yaw_margin),   NULL },

    { XPIEZO_PARAM_LAST,        NULL,                   {       0,       0,       0,       0, }, 0,       0, 0,                                                 NULL }
};


