/*
 * - Kp: P-Anteil Nick und Roll
 * - Kd: D-Anteil Nick und Roll
 * - angle_max: maximaler Winkel, der aufintegriert wird
 * - K-Stick: direkte lineare Stickwirkung auf den Winkel auf Nick und Roll
 * - Expo: (eigentlich x^2)-Anteil für Nick und Roll damit bei mehr stick
 *     weniger kontroller, dafür mehr speedunterschied auftritt
 * - K_expo_edge: ist die nicht-lineare Randverstärkung des Sticks für die
 *     Extrem-Turns (also ein Expo++ sozusagen)
 * - K_stick_direct: ist der Stick-Anteil für Nick und Roll der direkte am
 *     Winkel verbei wirkt ohne den ist das auf der Stelle schweben schwieriger
 * - Kp_yaw: P-Anteil Yaw
 * - Kd_Yaw: D-Anteil Yaw
 * - K_Stick_yaw: direkte lineare Stickwirkung auf den Winkel auf yaw
 * - fast yaw margin: gibt den Fast_Yaw_Mode in dem der Winkel igonieriert wird
 *     und der Margin bestimmt ab welcher Yaw-Speed der Winkel wieder aktiviert
 *     wird (0=erst bei Stillstand) solange wird nur der D abgebremst
 * - fast_yaw_enable macht das schaltet den Mode global ein und aus
 */

 /*
  * setting                     1         2         3         4
  *
  * kp                        220       178       178       180
  * kd                        470       428       428       470
  * angle_max             1000000   2000000   2000000   2000000
  * k_stick                     8        12        16        20
  * K_expo                     60        86       112       150
  * Kexpo_edge               1000      5000      9000     12000
  * K_stick_direkt             10        11        12        14
  *
  * kp_yaw                    410       392       392       400
  * kd_yaw                    535       514       514       540
  * K_stick_yaw                20        24        28        28
  * fast_yaw_margin            15        15        15        10
  * K_fast_yaw                  0        98       104       110
  * temp. comp.                60        50        50        50
  */

#ifndef __XPIEZO_H__
#define __XPIEZO_H__

#define XPIEZO_TYPE_PARAMETER_TYPE (xpiezo_parameter_type_get_type ())

GType xpiezo_parameter_type_get_type (void) G_GNUC_CONST;

#define XPIEZO_RESPONSE_PROGRESS 1
#define XPIEZO_RESPONSE_FINISHED 2

typedef enum _XPiezoParameter
{
  XPIEZO_EASYPARAMS,
  XPIEZO_EASY_WEIGHT,
  XPIEZO_EASY_AGILITY,

  XPIEZO_ORIENTATION,
  XPIEZO_TEMP_COMP,

  XPIEZO_NR_KP,
  XPIEZO_NR_KD,
  XPIEZO_NR_ANGLE_MAX,
  XPIEZO_NR_K_STICK,
  XPIEZO_NR_K_STICK_DIRECT,
  XPIEZO_NR_K_EXPO,
  XPIEZO_NR_K_EXPO_EDGE,

  XPIEZO_YAW_KP,
  XPIEZO_YAW_KD,
  XPIEZO_YAW_K_STICK,
  XPIEZO_YAW_FAST_YAW,
  XPIEZO_YAW_K_FAST,
  XPIEZO_YAW_FAST_MARGIN,

  XPIEZO_PARAM_LAST   /*< skip >*/
} XPiezoParameter;

typedef struct _XPiezoPreset XPiezoPreset;
typedef struct _XPiezoContext XPiezoContext;
typedef struct _ParamPreset ParamPreset;

typedef void (* XPiezoReadCallback) (XPiezoContext *context,
                                     gchar         *response,
                                     gsize          len);

struct _XPiezoPreset
{
  gboolean  easyparams;

  gint      orientation;
  gint      weight;   /* weight preset, lightweight to heavy, 0-7 */
  gint      agility;  /* lazy to agile, 0-7 */
  gint      temp_comp;

  /* expert parameters */

  /* Nick/Roll */
  gint      nr_kp;
  gint      nr_kd;
  gint      angle_max;
  gint      nr_k_stick;
  gint      nr_k_expo;
  gint      nr_k_expo_edge;
  gint      nr_k_stick_direct;

  /* Yaw */
  gint      yaw_kp;
  gint      yaw_kd;
  gint      yaw_k_stick;
  gboolean  fast_yaw;
  gint      fast_yaw_margin;
  gint      k_fast_yaw;
};

struct _XPiezoContext
{
  gchar               *serial_name;
  gint                 serial_fd;
  GIOChannel          *serial;

  guint                write_watch;
  gchar               *write_buf;
  gint                 write_n_bytes;
  gint                 written;

  XPiezoReadCallback   read_callback;
  guint                read_watch;
  gchar               *read_buf;
  gsize                read_buf_size;
  gsize                bytes_read;

  GtkWidget           *main_window;
  GdkPixbuf          **ufo_pixbufs;
  GtkWidget          **ori_buttons;
  GtkWidget           *fast_yaw_toggle;
  GtkWidget           *ufo_image;
  GtkWidget           *preset_button;
  GtkAction           *preset_actions[4];

  GtkWidget           *progress_dialog;
  gdouble              progress;
  GError              *error;

  gboolean             in_update;

  gint                 cur_preset;
  XPiezoPreset         presets[4];
  XPiezoPreset         tmp_presets[4];
};

struct _ParamPreset
{
  XPiezoParameter  param;
  gchar           *label;
  gint             factory[4];
  gdouble          min;
  gdouble          max;
  gint             s_offset;
  GtkAdjustment   *adj;
};

extern ParamPreset param_defs[];

void            xpiezo_preset_update_from_easy (XPiezoContext  *context);

gboolean        xpiezo_preset_save_to_file     (XPiezoPreset   *preset,
                                                gchar          *filename,
                                                GError        **error);
gboolean        xpiezo_config_save_to_file     (XPiezoContext  *context,
                                                gchar          *filename,
                                                GError        **error);
gboolean        xpiezo_x3d_init_config_load    (XPiezoContext  *context);
gboolean        xpiezo_x3d_init_config_write   (XPiezoContext  *context);

gboolean        xpiezo_preset_load_from_file   (XPiezoPreset   *preset,
                                                gchar          *filename,
                                                GError        **error);
gboolean        xpiezo_config_load_from_file   (XPiezoContext  *context,
                                                gchar          *filename,
                                                GError        **error);

GtkWidget      *xpiezo_create_window           (XPiezoContext  *context);
GtkActionGroup *xpiezo_get_actions             (XPiezoContext  *context);
void            xpiezo_refresh_gui             (XPiezoContext  *context);
void            xpiezo_config_defaults         (GtkAction      *action,
                                                XPiezoContext  *context);

void            xpiezo_close_piezo             (XPiezoContext  *context,
                                                GError        **error);
XPiezoPreset    xpiezo_preset_from_pst         (gchar          *pst_binary,
                                                gint            n_bytes);


#endif  /* __XPIEZO_H__ */

